//Chris Hunter
/******************HOPMAC*****************
 This exercise is an extension of the previous
 UNIMAC exercise. In fact, you are welcome
 top copy and paste the parts of the code you
 completed in that exercise for the similar
 tasks here.
 
 The twist in this lab is that the server
 will periodically hop to a new part of the
 spectrum. Before doing so, it will transmit
 a special control packet (HOPPACKET) that
 contains the destination frequency band.
 
 If the packet is missed, your receiver will
 be stuck until the transmitter randomly hops
 back onto the channel you were sitting at.
 This can potentially be a very long time.
 An advanced extension to this lab is to
 scan for packets in various channels when
 no good packet is received for a certain
 amount of time. For our purposes, let this
 timeout be around 200ms.
 *****************************************/

#include "warpmac.h"
#include "warpphy.h"
#include "xparameters.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "xutil.h"
#include "xcache_l.h"
#include "xexception_l.h"
#include "xcache_l.h"
#include "xutil.h"
#include "xexception_l.h"
#include "xtime_l.h"

#define ARPREQUEST 0x806

//Packet Types
#define DATAPACKET 0
#define ACKPACKET 1
#define HOPPACKET 2

//Routing table with agreed upon mapping between dipswitches and physical addresses
typedef struct {
	unsigned char addr[6];
} route;

//Byte array containing physical MAC address of this node
unsigned char myAddr[6];
//Instance of the routing table
route routeTable[16];
//Index to the routing table that identifies this node
unsigned char myID;


//Buffer for holding a packet-to-xmit across multiple retransmissions
Macframe txBuffer; 
//Buffer to hold received packet
Macframe rxBuffer;

unsigned char chan;
unsigned char controlMode = 1;

void middlebutton(){
	
	controlMode=0;
	
}

void upbutton(){
	
	controlMode=1;
	
}


void switchChannels(){
	Macframe followMe;
	followMe.isNew = 1;
							//	 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,11,12,13,14
	unsigned char chanVec[14] = {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0};
	//unsigned char chanVec[14] = {1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0};
	//unsigned char chanVec[14] = {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0};
	chan = rand()%14;
	while(chanVec[chan] != 1){//11
		chan = rand()%14;
	}
	chan = chan+1;

	
	unsigned char broadcast[6] = {255,255,255,255,255,255};
	followMe.header.length = 0;
	followMe.header.pktType = HOPPACKET;
	memcpy(followMe.header.destAddr,broadcast,6);
	followMe.header.reserved1 = chan;
	char i=0;
#define NUMCONTROL 8
	
	if(controlMode){
		while(i<NUMCONTROL){
			if(warpmac_carrierSense()) warpmac_sendOfdm(&followMe);
			i++;
		}
	}
	
	//	warpmac_sendOfdm(&txBuffer);
	//	warpmac_sendOfdm(&txBuffer);
	//	warpmac_sendOfdm(&txBuffer);
	//	warpmac_freePayload(&txBuffer);
	
		warpmac_setChannel(GHZ_2,chan);
		warpmac_leftHex(chan);
		//xil_printf("Chan = %d\r\n",chan);
	
	//	usleep(500);
	
	
}



int timerExpire(unsigned char timerType){
	/*This function is responsible for handling TIMEOUTs and BACKOFFs.
	It is registered using the warpmac_setTimerHandler function in
	main. The job responsibilities of this function are to:
	-increase the contention window upon the expiration of a TIMEOUT
	-initiate a BACKOFF timer upon the expiration of a TIMEOUT
	-retransmit a packet upon the expiration of a BACKOFF*/
	int status;
	switch(timerType){
		case TIMEOUT:
			if(txBuffer.isNew){
				status = warpmac_incrementResend(&txBuffer);
				if(status == 0){
					warpmac_enableEthernetInterrupt();
					return 0;
				}
				warpmac_setTimer(BACKOFF);
				return 0;
			}
			break;
		case BACKOFF:
			if(warpmac_carrierSense()){
				warpmac_sendOfdm(&txBuffer);
				warpmac_setTimer(TIMEOUT);
			}
			else{
				warpmac_setTimer(BACKOFF);
			}
			break;
			return 0;
	}
}

int ethernet_callback(Xuint32 length, char* payload){
	
	unsigned short int* pkttype = &payload[12];
	unsigned char* destNode;
	unsigned int* destIP;
	
	if(*pkttype==ARPREQUEST){
		destIP = &payload[38];
		destNode=&payload[41];
	}
	else{
		destIP = &payload[30];
		destNode=&payload[33];
	}
	
	if(*destIP>0xc0a80100 && *destIP<0xc0a8010F){
		warpmac_disableEthernetInterrupt();
		txBuffer.header.currReSend = 0;
		txBuffer.isNew = 1;
		txBuffer.header.length = length;
		txBuffer.header.pktType = DATAPACKET;
	
		//Set the modulation scheme for the packet's full-rate symbols
		txBuffer.header.fullRate = QPSK;
	
		//Copy in the packet's destination MAC address
		//Hard-coded for now as this node's partner node
		memcpy(txBuffer.header.destAddr,routeTable[*destNode].addr,6);
	
	
		if(warpmac_carrierSense()){
			warpmac_sendOfdm(&txBuffer);
			warpmac_setTimer(TIMEOUT);
		}
		else{
			warpmac_setTimer(BACKOFF);
		}
	}
		return 0;
}


int receiveBadPacket(Macframe* packet){
	/*This function blinks LEDs to show rate of bad packets.*/
	warpmac_incrementLEDLow();
}

int receiveGoodPacket(Macframe* packet) {
	warpmac_incrementLEDHigh();
	
	//IMPORTANT NOTE: Even though this function is passed a Macframe structure,
	//the data field is not yet filled. The user's MAC should process the packet
	//based on the header information, send out an ACK if necessary, and then
	//copy the payload out of the PHY with the warpmac_writePacket command.
	
	if(warpmac_addressedToMe(packet)){
		Macframe ackPacket;
		switch(packet->header.pktType){
			case ACKPACKET:
				if(warpmac_inTimeout()){
					warpmac_clearTimer(TIMEOUT);
					txBuffer.header.currReSend = 0;
					txBuffer.isNew = 0;
					warpmac_enableEthernetInterrupt();
				}
				break;
			case DATAPACKET:
								
				packet->header.currReSend = 0;
				packet->isNew = 1;
				warpmac_phyInterruptClear();
				warpmac_sendEthernet(packet);
				packet->header.currReSend = 0;
				packet->isNew = 0;
				
			break;

		}
	}
return 0;

}


int main(){
	//Initialize the framework
	unsigned char antSel = 0;
	
	//Read Dip Switch value from FPGA board.
	//This value will be used as an index into the routing table for other nodes
	myID = warpmac_getMyId();
	
	//Create an arbitrary address for this node
	unsigned char tmpAddr[6] = {0x16,0x24,0x63,0x53,0xe2,0xc2+myID};
	
	memcpy(myAddr,tmpAddr,6);
	
	//Fill an arbitrary routing table so that nodes know each others' addresses
	unsigned char i;
	for(i=0;i<16;i++){
		routeTable[i].addr[0] = myAddr[0];
		routeTable[i].addr[1] = myAddr[1];
		routeTable[i].addr[2] = myAddr[2];
		routeTable[i].addr[3] = myAddr[3];
		routeTable[i].addr[4] = myAddr[4];
		routeTable[i].addr[5] = myAddr[5]+i-myID;
	}
	
	//Initialize the framework
	warpmac_init();
	
	warpmac_setMacAddr(&myAddr);
	warpmac_setMaxResend(8);
	warpmac_setMaxCW(5);
	warpmac_setTimeout(160);
	warpmac_setSlotTime(9);
	
	warpmac_setRxBuffer(&rxBuffer,0);
	warpmac_setTxBuffer(1);
	
 	//Copy MAC address into source field
	memcpy(txBuffer.header.srcAddr,myAddr,6);
	
	warpmac_setGoodPacketHandler(receiveGoodPacket);
	warpmac_setBadPacketHandler(receiveBadPacket);
	
	warpmac_setTimerHandler(timerExpire);
	warpmac_setEmacHandler(ethernet_callback);
	
	warpmac_setMiddleButtonHandler(middlebutton);
	warpmac_setUpButtonHandler(upbutton);

	//Set the RF center frequency
	xil_printf("Channel = %d\r\n",warpmac_setChannel(GHZ_2,8));
	
	warpmac_enableCSMA();
	warpmac_enableEthernetInterrupt();
	
	//Set the modulation scheme use for base rate (header) symbols
	// Every node must use the same mod scheme for base rate symbols!
	warpmac_setBaseRate(QPSK);
	
	XTime tEnd, tCur;
	unsigned int seconds = 5;
	unsigned char startCount = 1;
	
	xil_printf("Mask version 1\r\n");
	while(1){
		XTime_GetTime(&tCur);
		usleep(1000);
		if (startCount){
			tEnd  = tCur + ((XTime) seconds) * 160000000;
			startCount=0;
		}
		
		//	if(tCur>tEnd){
		//		switchChannels();
		//		startCount=1;
		//		seconds = 1;
		//
		//	}
		if(tCur>tEnd){
			if(txBuffer.isNew==0){	
				warpmac_disableEthernetInterrupt();
				usleep(1000);
				switchChannels();
				startCount=1;
				seconds = 5;
				txBuffer.isNew = 0;
				warpmac_enableEthernetInterrupt();
			}
		}
	}
}




